@@ -1,5 +1,5 @@
 /******************************************************************************
-    Copyright (C) 2013 by Hugh Bailey <obs.jim@gmail.com>
+    Copyright (C) 2023 by Lain Bailey <lain@obsproject.com>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,11 +25,6 @@
 #include "gl-subsystem.h"
 #include "gl-shaderparser.h"
 
-static inline void shader_param_init(struct gs_shader_param *param)
-{
-	memset(param, 0, sizeof(struct gs_shader_param));
-}
-
 static inline void shader_param_free(struct gs_shader_param *param)
 {
 	bfree(param->name);
@@ -42,18 +37,17 @@ static inline void shader_attrib_free(struct shader_attrib *attrib)
 	bfree(attrib->name);
 }
 
-static void gl_get_shader_info(GLuint shader, const char *file,
-		char **error_string)
+static void gl_get_shader_info(GLuint shader, const char *file, char **error_string)
 {
-	char    *errors;
-	GLint   info_len = 0;
+	char *errors;
+	GLint info_len = 0;
 	GLsizei chars_written = 0;
 
 	glGetShaderiv(shader, GL_INFO_LOG_LENGTH, &info_len);
 	if (!gl_success("glGetProgramiv") || !info_len)
 		return;
 
-	errors = bzalloc(info_len+1);
+	errors = bzalloc(info_len + 1);
 	glGetShaderInfoLog(shader, info_len, &chars_written, errors);
 	gl_success("glGetShaderInfoLog");
 
@@ -65,19 +59,18 @@ static void gl_get_shader_info(GLuint shader, const char *file,
 		bfree(errors);
 }
 
-static bool gl_add_param(struct gs_shader *shader, struct shader_var *var,
-		GLint *texture_id)
+static bool gl_add_param(struct gs_shader *shader, struct shader_var *var, GLint *texture_id)
 {
 	struct gs_shader_param param = {0};
 
 	param.array_count = var->array_count;
-	param.name        = bstrdup(var->name);
-	param.shader      = shader;
-	param.type        = get_shader_param_type(var->type);
+	param.name = bstrdup(var->name);
+	param.shader = shader;
+	param.type = get_shader_param_type(var->type);
 
 	if (param.type == GS_SHADER_PARAM_TEXTURE) {
-		param.sampler_id  = var->gl_sampler_id;
-		param.texture_id  = (*texture_id)++;
+		param.sampler_id = var->gl_sampler_id;
+		param.texture_id = (*texture_id)++;
 	} else {
 		param.changed = true;
 	}
@@ -89,24 +82,22 @@ static bool gl_add_param(struct gs_shader *shader, struct shader_var *var,
 	return true;
 }
 
-static inline bool gl_add_params(struct gs_shader *shader,
-		struct gl_shader_parser *glsp)
+static inline bool gl_add_params(struct gs_shader *shader, struct gl_shader_parser *glsp)
 {
 	size_t i;
 	GLint tex_id = 0;
 
 	for (i = 0; i < glsp->parser.params.num; i++)
-		if (!gl_add_param(shader, glsp->parser.params.array+i, &tex_id))
+		if (!gl_add_param(shader, glsp->parser.params.array + i, &tex_id))
 			return false;
 
 	shader->viewproj = gs_shader_get_param_by_name(shader, "ViewProj");
-	shader->world    = gs_shader_get_param_by_name(shader, "World");
+	shader->world = gs_shader_get_param_by_name(shader, "World");
 
 	return true;
 }
 
-static inline void gl_add_sampler(struct gs_shader *shader,
-		struct shader_sampler *sampler)
+static inline void gl_add_sampler(struct gs_shader *shader, struct shader_sampler *sampler)
 {
 	gs_samplerstate_t *new_sampler;
 	struct gs_sampler_info info;
@@ -117,45 +108,42 @@ static inline void gl_add_sampler(struct gs_shader *shader,
 	da_push_back(shader->samplers, &new_sampler);
 }
 
-static inline void gl_add_samplers(struct gs_shader *shader,
-		struct gl_shader_parser *glsp)
+static inline void gl_add_samplers(struct gs_shader *shader, struct gl_shader_parser *glsp)
 {
 	size_t i;
 	for (i = 0; i < glsp->parser.samplers.num; i++) {
-		struct shader_sampler *sampler = glsp->parser.samplers.array+i;
+		struct shader_sampler *sampler = glsp->parser.samplers.array + i;
 		gl_add_sampler(shader, sampler);
 	}
 }
 
-static void get_attrib_type(const char *mapping, enum attrib_type *type,
-		size_t *index)
+static void get_attrib_type(const char *mapping, enum attrib_type *type, size_t *index)
 {
 	if (strcmp(mapping, "POSITION") == 0) {
-		*type  = ATTRIB_POSITION;
+		*type = ATTRIB_POSITION;
 
 	} else if (strcmp(mapping, "NORMAL") == 0) {
-		*type  = ATTRIB_NORMAL;
+		*type = ATTRIB_NORMAL;
 
 	} else if (strcmp(mapping, "TANGENT") == 0) {
-		*type  = ATTRIB_TANGENT;
+		*type = ATTRIB_TANGENT;
 
 	} else if (strcmp(mapping, "COLOR") == 0) {
-		*type  = ATTRIB_COLOR;
+		*type = ATTRIB_COLOR;
 
 	} else if (astrcmp_n(mapping, "TEXCOORD", 8) == 0) {
-		*type  = ATTRIB_TEXCOORD;
-		*index = (*(mapping+8)) - '0';
+		*type = ATTRIB_TEXCOORD;
+		*index = (*(mapping + 8)) - '0';
 		return;
 
 	} else if (strcmp(mapping, "TARGET") == 0) {
-		*type  = ATTRIB_TARGET;
+		*type = ATTRIB_TARGET;
 	}
 
 	*index = 0;
 }
 
-static inline bool gl_process_attrib(struct gs_shader *program,
-		struct gl_parser_attrib *pa)
+static inline bool gl_process_attrib(struct gs_shader *program, struct gl_parser_attrib *pa)
 {
 	struct shader_attrib attrib = {0};
 
@@ -166,20 +154,19 @@ static inline bool gl_process_attrib(struct gs_shader *program,
 	get_attrib_type(pa->mapping, &attrib.type, &attrib.index);
 	attrib.name = pa->name.array;
 
-	pa->name.array    = NULL;
-	pa->name.len      = 0;
+	pa->name.array = NULL;
+	pa->name.len = 0;
 	pa->name.capacity = 0;
 
 	da_push_back(program->attribs, &attrib);
 	return true;
 }
 
-static inline bool gl_process_attribs(struct gs_shader *shader,
-		struct gl_shader_parser *glsp)
+static inline bool gl_process_attribs(struct gs_shader *shader, struct gl_shader_parser *glsp)
 {
 	size_t i;
 	for (i = 0; i < glsp->attribs.num; i++) {
-		struct gl_parser_attrib *pa = glsp->attribs.array+i;
+		struct gl_parser_attrib *pa = glsp->attribs.array + i;
 		if (!gl_process_attrib(shader, pa))
 			return false;
 	}
@@ -187,9 +174,8 @@ static inline bool gl_process_attribs(struct gs_shader *shader,
 	return true;
 }
 
-static bool gl_shader_init(struct gs_shader *shader,
-		struct gl_shader_parser *glsp,
-		const char *file, char **error_string)
+static bool gl_shader_init(struct gs_shader *shader, struct gl_shader_parser *glsp, const char *file,
+			   char **error_string)
 {
 	GLenum type = convert_shader_type(shader->type);
 	int compiled = 0;
@@ -199,8 +185,7 @@ static bool gl_shader_init(struct gs_shader *shader,
 	if (!gl_success("glCreateShader") || !shader->obj)
 		return false;
 
-	glShaderSource(shader->obj, 1, (const GLchar**)&glsp->gl_string.array,
-			0);
+	glShaderSource(shader->obj, 1, (const GLchar **)&glsp->gl_string.array, 0);
 	if (!gl_success("glShaderSource"))
 		return false;
 
@@ -220,8 +205,22 @@ static bool gl_shader_init(struct gs_shader *shader,
 	if (!gl_success("glGetShaderiv"))
 		return false;
 
-	if (!compiled)
+	if (!compiled) {
+		GLint infoLength = 0;
+		glGetShaderiv(shader->obj, GL_INFO_LOG_LENGTH, &infoLength);
+
+		char *infoLog = malloc(sizeof(char) * infoLength);
+
+		if (infoLog) {
+			GLsizei returnedLength = 0;
+			glGetShaderInfoLog(shader->obj, infoLength, &returnedLength, infoLog);
+			blog(LOG_ERROR, "Error compiling shader:\n%s\n", infoLog);
+
+			free(infoLog);
+		}
+
 		success = false;
+	}
 
 	gl_get_shader_info(shader->obj, file, error_string);
 
@@ -236,16 +235,15 @@ static bool gl_shader_init(struct gs_shader *shader,
 	return success;
 }
 
-static struct gs_shader *shader_create(gs_device_t *device,
-		enum gs_shader_type type, const char *shader_str,
-		const char *file, char **error_string)
+static struct gs_shader *shader_create(gs_device_t *device, enum gs_shader_type type, const char *shader_str,
+				       const char *file, char **error_string)
 {
 	struct gs_shader *shader = bzalloc(sizeof(struct gs_shader));
 	struct gl_shader_parser glsp;
 	bool success = true;
 
 	shader->device = device;
-	shader->type   = type;
+	shader->type = type;
 
 	gl_shader_parser_init(&glsp, type);
 	if (!gl_shader_parse(&glsp, shader_str, file))
@@ -262,25 +260,19 @@ static struct gs_shader *shader_create(gs_device_t *device,
 	return shader;
 }
 
-gs_shader_t *device_vertexshader_create(gs_device_t *device,
-		const char *shader, const char *file,
-		char **error_string)
+gs_shader_t *device_vertexshader_create(gs_device_t *device, const char *shader, const char *file, char **error_string)
 {
 	struct gs_shader *ptr;
-	ptr = shader_create(device, GS_SHADER_VERTEX, shader, file,
-			error_string);
+	ptr = shader_create(device, GS_SHADER_VERTEX, shader, file, error_string);
 	if (!ptr)
 		blog(LOG_ERROR, "device_vertexshader_create (GL) failed");
 	return ptr;
 }
 
-gs_shader_t *device_pixelshader_create(gs_device_t *device,
-		const char *shader, const char *file,
-		char **error_string)
+gs_shader_t *device_pixelshader_create(gs_device_t *device, const char *shader, const char *file, char **error_string)
 {
 	struct gs_shader *ptr;
-	ptr = shader_create(device, GS_SHADER_PIXEL, shader, file,
-			error_string);
+	ptr = shader_create(device, GS_SHADER_PIXEL, shader, file, error_string);
 	if (!ptr)
 		blog(LOG_ERROR, "device_pixelshader_create (GL) failed");
 	return ptr;
@@ -294,12 +286,10 @@ static void remove_program_references(struct gs_shader *shader)
 		struct gs_program *next = program->next;
 		bool destroy = false;
 
-		if (shader->type == GS_SHADER_VERTEX &&
-				program->vertex_shader == shader)
+		if (shader->type == GS_SHADER_VERTEX && program->vertex_shader == shader)
 			destroy = true;
 
-		else if (shader->type == GS_SHADER_PIXEL &&
-				program->pixel_shader == shader)
+		else if (shader->type == GS_SHADER_PIXEL && program->pixel_shader == shader)
 			destroy = true;
 
 		if (destroy)
@@ -319,13 +309,13 @@ void gs_shader_destroy(gs_shader_t *shader)
 	remove_program_references(shader);
 
 	for (i = 0; i < shader->attribs.num; i++)
-		shader_attrib_free(shader->attribs.array+i);
+		shader_attrib_free(shader->attribs.array + i);
 
 	for (i = 0; i < shader->samplers.num; i++)
 		gs_samplerstate_destroy(shader->samplers.array[i]);
 
 	for (i = 0; i < shader->params.num; i++)
-		shader_param_free(shader->params.array+i);
+		shader_param_free(shader->params.array + i);
 
 	if (shader->obj) {
 		glDeleteShader(shader->obj);
@@ -346,14 +336,14 @@ int gs_shader_get_num_params(const gs_shader_t *shader)
 gs_sparam_t *gs_shader_get_param_by_idx(gs_shader_t *shader, uint32_t param)
 {
 	assert(param < shader->params.num);
-	return shader->params.array+param;
+	return shader->params.array + param;
 }
 
 gs_sparam_t *gs_shader_get_param_by_name(gs_shader_t *shader, const char *name)
 {
 	size_t i;
 	for (i = 0; i < shader->params.num; i++) {
-		struct gs_shader_param *param = shader->params.array+i;
+		struct gs_shader_param *param = shader->params.array + i;
 
 		if (strcmp(param->name, name) == 0)
 			return param;
@@ -372,8 +362,7 @@ gs_sparam_t *gs_shader_get_world_matrix(const gs_shader_t *shader)
 	return shader->world;
 }
 
-void gs_shader_get_param_info(const gs_sparam_t *param,
-		struct gs_shader_param_info *info)
+void gs_shader_get_param_info(const gs_sparam_t *param, struct gs_shader_param_info *info)
 {
 	info->type = param->type;
 	info->name = param->name;
@@ -428,74 +417,88 @@ void gs_shader_set_texture(gs_sparam_t *param, gs_texture_t *val)
 	param->texture = val;
 }
 
-static inline bool validate_param(struct program_param *pp,
-		size_t expected_size)
+static inline bool validate_param(struct program_param *pp, size_t expected_size)
 {
 	if (pp->param->cur_value.num != expected_size) {
-		blog(LOG_ERROR, "Parameter '%s' set to invalid size %u, "
-		                "expected %u",
-		                pp->param->name,
-		                (unsigned int)pp->param->cur_value.num,
-		                (unsigned int)expected_size);
+		blog(LOG_ERROR,
+		     "Parameter '%s' set to invalid size %u, "
+		     "expected %u",
+		     pp->param->name, (unsigned int)pp->param->cur_value.num, (unsigned int)expected_size);
 		return false;
 	}
 
 	return true;
 }
 
-static void program_set_param_data(struct gs_program *program,
-		struct program_param *pp)
+static void program_set_param_data(struct gs_program *program, struct program_param *pp)
 {
 	void *array = pp->param->cur_value.array;
 
-	if (pp->param->type == GS_SHADER_PARAM_BOOL ||
-	    pp->param->type == GS_SHADER_PARAM_INT) {
+	if (pp->param->type == GS_SHADER_PARAM_BOOL || pp->param->type == GS_SHADER_PARAM_INT) {
 		if (validate_param(pp, sizeof(int))) {
-			glUniform1iv(pp->obj, 1, (int*)array);
+			glUniform1iv(pp->obj, 1, (int *)array);
 			gl_success("glUniform1iv");
 		}
 
+	} else if (pp->param->type == GS_SHADER_PARAM_INT2) {
+		if (validate_param(pp, sizeof(int) * 2)) {
+			glUniform2iv(pp->obj, 1, (int *)array);
+			gl_success("glUniform2iv");
+		}
+
+	} else if (pp->param->type == GS_SHADER_PARAM_INT3) {
+		if (validate_param(pp, sizeof(int) * 3)) {
+			glUniform3iv(pp->obj, 1, (int *)array);
+			gl_success("glUniform3iv");
+		}
+
+	} else if (pp->param->type == GS_SHADER_PARAM_INT4) {
+		if (validate_param(pp, sizeof(int) * 4)) {
+			glUniform4iv(pp->obj, 1, (int *)array);
+			gl_success("glUniform4iv");
+		}
+
 	} else if (pp->param->type == GS_SHADER_PARAM_FLOAT) {
 		if (validate_param(pp, sizeof(float))) {
-			glUniform1fv(pp->obj, 1, (float*)array);
+			glUniform1fv(pp->obj, 1, (float *)array);
 			gl_success("glUniform1fv");
 		}
 
 	} else if (pp->param->type == GS_SHADER_PARAM_VEC2) {
 		if (validate_param(pp, sizeof(struct vec2))) {
-			glUniform2fv(pp->obj, 1, (float*)array);
+			glUniform2fv(pp->obj, 1, (float *)array);
 			gl_success("glUniform2fv");
 		}
 
 	} else if (pp->param->type == GS_SHADER_PARAM_VEC3) {
 		if (validate_param(pp, sizeof(float) * 3)) {
-			glUniform3fv(pp->obj, 1, (float*)array);
+			glUniform3fv(pp->obj, 1, (float *)array);
 			gl_success("glUniform3fv");
 		}
 
 	} else if (pp->param->type == GS_SHADER_PARAM_VEC4) {
 		if (validate_param(pp, sizeof(struct vec4))) {
-			glUniform4fv(pp->obj, 1, (float*)array);
+			glUniform4fv(pp->obj, 1, (float *)array);
 			gl_success("glUniform4fv");
 		}
 
 	} else if (pp->param->type == GS_SHADER_PARAM_MATRIX4X4) {
 		if (validate_param(pp, sizeof(struct matrix4))) {
-			glUniformMatrix4fv(pp->obj, 1, false,
-					(float*)array);
+			glUniformMatrix4fv(pp->obj, 1, false, (float *)array);
 			gl_success("glUniformMatrix4fv");
 		}
 
 	} else if (pp->param->type == GS_SHADER_PARAM_TEXTURE) {
 		if (pp->param->next_sampler) {
-			program->device->cur_samplers[pp->param->sampler_id] =
-				pp->param->next_sampler;
+			program->device->cur_samplers[pp->param->sampler_id] = pp->param->next_sampler;
 			pp->param->next_sampler = NULL;
 		}
 
 		glUniform1i(pp->obj, pp->param->texture_id);
-		device_load_texture(program->device, pp->param->texture,
-				pp->param->texture_id);
+		if (pp->param->srgb)
+			device_load_texture_srgb(program->device, pp->param->texture, pp->param->texture_id);
+		else
+			device_load_texture(program->device, pp->param->texture, pp->param->texture_id);
 	}
 }
 
@@ -509,15 +512,15 @@ void program_update_params(struct gs_program *program)
 
 static void print_link_errors(GLuint program)
 {
-	char    *errors = NULL;
-	GLint   info_len = 0;
+	char *errors = NULL;
+	GLint info_len = 0;
 	GLsizei chars_written = 0;
 
 	glGetProgramiv(program, GL_INFO_LOG_LENGTH, &info_len);
 	if (!gl_success("glGetProgramiv") || !info_len)
 		return;
 
-	errors = calloc(1, info_len+1);
+	errors = calloc(1, info_len + 1);
 	glGetProgramInfoLog(program, info_len, &chars_written, errors);
 	gl_success("glGetShaderInfoLog");
 
@@ -526,16 +529,17 @@ static void print_link_errors(GLuint program)
 	free(errors);
 }
 
-static bool assign_program_attrib(struct gs_program *program,
-		struct shader_attrib *attrib)
+static bool assign_program_attrib(struct gs_program *program, struct shader_attrib *attrib)
 {
 	GLint attrib_obj = glGetAttribLocation(program->obj, attrib->name);
 	if (!gl_success("glGetAttribLocation"))
 		return false;
 
 	if (attrib_obj == -1) {
-		blog(LOG_ERROR, "glGetAttribLocation: Could not find "
-		                "attribute '%s'", attrib->name);
+		blog(LOG_ERROR,
+		     "glGetAttribLocation: Could not find "
+		     "attribute '%s'",
+		     attrib->name);
 		return false;
 	}
 
@@ -556,8 +560,7 @@ static inline bool assign_program_attribs(struct gs_program *program)
 	return true;
 }
 
-static bool assign_program_param(struct gs_program *program,
-		struct gs_shader_param *param)
+static bool assign_program_param(struct gs_program *program, struct gs_shader_param *param)
 {
 	struct program_param info;
 
@@ -574,8 +577,7 @@ static bool assign_program_param(struct gs_program *program,
 	return true;
 }
 
-static inline bool assign_program_shader_params(struct gs_program *program,
-		struct gs_shader *shader)
+static inline bool assign_program_shader_params(struct gs_program *program, struct gs_shader *shader)
 {
 	for (size_t i = 0; i < shader->params.num; i++) {
 		struct gs_shader_param *param = shader->params.array + i;
@@ -601,9 +603,9 @@ struct gs_program *gs_program_create(struct gs_device *device)
 	struct gs_program *program = bzalloc(sizeof(*program));
 	int linked = false;
 
-	program->device        = device;
+	program->device = device;
 	program->vertex_shader = device->cur_vertex_shader;
-	program->pixel_shader  = device->cur_pixel_shader;
+	program->pixel_shader = device->cur_pixel_shader;
 
 	program->obj = glCreateProgram();
 	if (!gl_success("glCreateProgram"))
@@ -694,16 +696,40 @@ void gs_shader_set_val(gs_sparam_t *param, const void *val, size_t size)
 	if (!count)
 		count = 1;
 
-	switch ((uint32_t)param->type) {
-	case GS_SHADER_PARAM_FLOAT:     expected_size = sizeof(float); break;
+	switch (param->type) {
+	case GS_SHADER_PARAM_FLOAT:
+		expected_size = sizeof(float);
+		break;
 	case GS_SHADER_PARAM_BOOL:
-	case GS_SHADER_PARAM_INT:       expected_size = sizeof(int); break;
-	case GS_SHADER_PARAM_VEC2:      expected_size = sizeof(float)*2; break;
-	case GS_SHADER_PARAM_VEC3:      expected_size = sizeof(float)*3; break;
-	case GS_SHADER_PARAM_VEC4:      expected_size = sizeof(float)*4; break;
-	case GS_SHADER_PARAM_MATRIX4X4: expected_size = sizeof(float)*4*4;break;
-	case GS_SHADER_PARAM_TEXTURE:   expected_size = sizeof(void*); break;
-	default:                        expected_size = 0;
+	case GS_SHADER_PARAM_INT:
+		expected_size = sizeof(int);
+		break;
+	case GS_SHADER_PARAM_INT2:
+		expected_size = sizeof(int) * 2;
+		break;
+	case GS_SHADER_PARAM_INT3:
+		expected_size = sizeof(int) * 3;
+		break;
+	case GS_SHADER_PARAM_INT4:
+		expected_size = sizeof(int) * 4;
+		break;
+	case GS_SHADER_PARAM_VEC2:
+		expected_size = sizeof(float) * 2;
+		break;
+	case GS_SHADER_PARAM_VEC3:
+		expected_size = sizeof(float) * 3;
+		break;
+	case GS_SHADER_PARAM_VEC4:
+		expected_size = sizeof(float) * 4;
+		break;
+	case GS_SHADER_PARAM_MATRIX4X4:
+		expected_size = sizeof(float) * 4 * 4;
+		break;
+	case GS_SHADER_PARAM_TEXTURE:
+		expected_size = sizeof(struct gs_shader_texture);
+		break;
+	default:
+		expected_size = 0;
 	}
 
 	expected_size *= count;
@@ -712,14 +738,18 @@ void gs_shader_set_val(gs_sparam_t *param, const void *val, size_t size)
 
 	if (expected_size != size) {
 		blog(LOG_ERROR, "gs_shader_set_val (GL): Size of shader "
-		                "param does not match the size of the input");
+				"param does not match the size of the input");
 		return;
 	}
 
-	if (param->type == GS_SHADER_PARAM_TEXTURE)
-		gs_shader_set_texture(param, *(gs_texture_t**)val);
-	else
+	if (param->type == GS_SHADER_PARAM_TEXTURE) {
+		struct gs_shader_texture shader_tex;
+		memcpy(&shader_tex, val, sizeof(shader_tex));
+		gs_shader_set_texture(param, shader_tex.tex);
+		param->srgb = shader_tex.srgb;
+	} else {
 		da_copy_array(param->cur_value, val, size);
+	}
 }
 
 void gs_shader_set_default(gs_sparam_t *param)
