@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013 Hugh Bailey <obs.jim@gmail.com>
+ * Copyright (c) 2023 Lain Bailey <lain@obsproject.com>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -20,8 +20,5 @@ struct HRError {
 	const char *str;
 	HRESULT hr;
 
-	inline HRError(const char *str, HRESULT hr)
-		: str(str), hr (hr)
-	{
-	}
+	inline HRError(const char *str, HRESULT hr) : str(str), hr(hr) {}
 };
